/* df3.h */

#ifndef DF3_LIBRARY_H

#define DF3_LIBVERSION 201705L
#define DF3_LIBRARY_H  DF3_LIBVERSION

#include <inttypes.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

/* mandatory extension */
#define DF3_EXTENSION  ".df3"

/* df3Error(s) */
enum df3Error {
  ecOk = 0,
  ecBadArgument,
  ecBadDimsAxes,
  ecBadDimsBpv,
  ecBadHandle,
  ecBadVoxAt,
  ecBadVoxVal,
  ecNotDf3File,
  ecUnusableFile,
  ecSysErr,
  ecLastEC
};
typedef enum df3Error df3Error;

/* df3Axes specify a volume's extents.
 * df3Coord is an alias used for voxel addressing.
 */
typedef struct df3Axes df3Axes, df3Coord;
struct df3Axes { size_t x, y, z; };

/* df3Handle represents an open volume.
 * note that only the first three members are public,
 * the remainder of the structure is opaque, and its
 * size and members may change in a future release.
 * ne touche pas.  from after a successful call to
 * df3_open_file or df3_create_file until the call to
 * df3_close_file, the fname, axes and bpv members
 * can be read, but must not be changed.
 */
typedef struct df3Handle df3Handle;
struct df3Handle {
  char *fname;   /* fname, ie the file name, is expected to
                  * have the mandatory '.df3' extension,
                  * adhere to the POSIX.1 specifications,
                  * and to have been canonicalised using
                  * realpath(3) or similar.
                  */
  df3Axes axes;  /* axes gives the volume's extents,
                  * range: 1 <= x,y,z <= 65535.
                  */
  size_t bpv;    /* bytes per voxel is the number of octets
                  * in the DF3 data word, ie 1, 2, or 4.
                  */
  bool readonly;
  bool ismapped;
  void *dp;
  uint16_t chcks;
  int syserr;
  df3Error ec;
};
/* initialising a handle */
#define DF3_INITHANDLE \
    (df3Handle){NULL, {0, 0, 0}, 0, false, false, NULL, 0, 0, ecOk}

/* functions, see "man 3 df3" */
#define AWUR  __attribute__((warn_unused_result))
df3Error df3_open_file(df3Handle *, bool const) AWUR;
df3Error df3_create_file(df3Handle *) AWUR;
df3Error df3_close_file(df3Handle *);
df3Error df3_get_voxel(df3Handle *, df3Coord const, uint32_t *);
df3Error df3_put_voxel(df3Handle *, df3Coord const, uint32_t const);
bool df3_check_axes(df3Axes const);
bool df3_check_bpv(size_t const);
bool df3_check_fname(char const * const);
void df3_handle_info(df3Handle const * const, FILE *);
char const *df3_handle_error(df3Handle *, bool const);
char const *df3_error_message(df3Error const);

#endif

/* ----------------------------------------------------------------- *
 *  this file is part of 'libdf3', part of the 'df3 tools' project.  *
 *  the content above is covered by the GPLv3+, see file COPYING.    *
 *  copyright (c) 2017 jr <creature.eternal@gmail.com>.              *
 *  all rights reserved.                                             *
 * ----------------------------------------------------------------- */
