/* libdf3 - voxget.c */

#include "df3_priv.h"

#include <arpa/inet.h>
#include <string.h>


df3Error df3_get_voxel(df3Handle *df3, df3Coord const at, uint32_t *val) {
  if (!df3  ||  !df3ValidHandle(df3))
    return ecBadHandle;
  else if (!df3ValidAt(df3->axes, at))
    return (df3->ec = ecBadDimsAxes);

  off_t const x = df3CalcFilepointer(df3, at);
  endianData u = {{0}};

  if (df3->ismapped) {
    mmFile const * const p = df3->dp;
    u.v8 = p->dp[(size_t)x];
  }
  else if (-1 == fseek(df3->dp, x, SEEK_SET)) {
    df3->syserr = errno;
    return (df3->ec = ecSysErr);
  }
  else if (df3->bpv != fread(&u.raw, sizeof(uint8_t), df3->bpv, df3->dp)) {
    df3->syserr = errno;
    return (df3->ec = ecSysErr);
  }

  switch (df3->bpv) {
    case 4: *val = ntohl(u.v32); break;
    case 2: *val = ntohs(u.v16); break;
    case 1: *val = u.v8;         break;
  }

  return ecOk;
}

/* ----------------------------------------------------------------- *
 *  this file is part of 'libdf3', part of the 'df3 tools' project.  *
 *  the content above is covered by the GPLv3+, see file COPYING.    *
 *  copyright (c) 2017 jr <creature.eternal@gmail.com>.              *
 *  all rights reserved.                                             *
 * ----------------------------------------------------------------- */
